import pandas as pd


class InstituteData:
    def __init__(self, pd_dataframe):
        self._pd_dataframe = pd_dataframe
        self._id = None
        self._name = None
        self._name_abbr = None
        self._department = None
        self._department_abbr = None
        self._address = None
        self._city = None
        self._country = None
        self._coordinates = None
        self._url = None
        self._load_institute()

    @property
    def name(self):
        return self._name

    @property
    def id(self):
        return self._id

    @property
    def name(self):
        return self._name

    @property
    def name_abbr(self):
        return self._name_abbr
    
    @property
    def department(self):
        return self._department
    
    @property
    def department_abbr(self):
        return self._department_abbr

    @property
    def address(self):
        return self._address

    @property
    def city(self):
        return self._city

    @property
    def country(self):
        return self._country

    @property
    def coordinates(self):
        return self._coordinates

    @property
    def url(self):
        return self._url

    def convert_coordinates(self, lalng):
        lalng = lalng.replace("°", "").strip()
        if "S" in lalng or "W" in lalng:
            lalng = lalng.replace("S", "").replace("W", "")
            lalng = "-" + lalng
        else:
            lalng = lalng.replace("N", "").replace("E", "")
        return float(lalng)

    def process_coordinates(self):
        # Coordinates can be of the form 37.7983° S, 144.9610° E or 3.342119819025848, -76.5306449189542
        # We will convert them to the form 37.7983, 144.9610
        if self._coordinates is not None:
            # split into latitude and longitude
            coordinates = self._coordinates.split(", ")
            coordinates = [self.convert_coordinates(
                coord) for coord in coordinates]
            self._coordinates = tuple(coordinates)

    def _load_institute(self):
        self._id = self._pd_dataframe["id"]
        self._name = self._pd_dataframe["Name"]
        self._name_abbr = self._pd_dataframe["Name Abbreviation"]
        self._department = self._pd_dataframe["Department"]
        self._department_abbr = self._pd_dataframe["Department Abbreviation"]
        self._address = self._pd_dataframe["Address"]
        self._city = self._pd_dataframe["City"]
        self._country = self._pd_dataframe["Country"]
        self._url = self._pd_dataframe["URL"]
        self._coordinates = self._pd_dataframe["Coordinates"]
        self.process_coordinates()

    def __str__(self):
        string = f"Institute ID: {self._id}\n"
        string += f"Institute Name: {self._name}\n"
        string += f"Institute Name Abbreviation: {self._name_abbr}\n"
        string += f"Institute Department: {self._department}\n"
        string += f"Institute Department Abbreviation: {self._department_abbr}\n"
        string += f"Institute Address: {self._address}\n"
        string += f"Institute City: {self._city}\n"
        string += f"Institute Country: {self._country}\n"
        string += f"Institute URL: {self._url}\n"
        string += f"Institute Coordinates: {self._coordinates}\n\n"
        return string

    def __repr__(self):
        string = f"InstituteData(id={self._id}, name={self._name}, name_abbr={self._name_abbr}, address={self._address}, city={self._city}, country={self._country}, url={self._url}, coordinates={self._coordinates})\n"
        return string


class Institutes:
    def __init__(self, filename):
        self._filename = filename
        self._institute = self._load_institutes()

    @property
    def filename(self):
        return self._filename

    @property
    def institute(self):
        return self._institute

    def get_institute(self, institute_id):
        if isinstance(institute_id, str):
            institute_id = int(institute_id)
        for institute in self.institute:
            if institute.id == institute_id:
                return institute
        return None

    def _load_institutes(self):
        institutes_df = pd.read_excel(self.filename, sheet_name="Institutes")
        return [InstituteData(institute) for index, institute in institutes_df.iterrows()]

    def __str__(self):
        string = ""
        for institute in self.institute:
            string += str(institute)
        return string

    def __repr__(self):
        repr = f"Institutes(filename={self.filename}, institute={self.institute})\n"
        return repr
